@@ -100,6 +100,7 @@
  *	automated updates after select() (i always missed that with OSS!).
  */
 
+#define _GNU_SOURCE
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -193,6 +194,8 @@ static char	 card_id[64] = "default";
 static snd_mixer_t *mixer_handle;
 static char	 mixer_card_name[128];
 static char	 mixer_device_name[128];
+static int	 mixer_level = 0;
+static struct snd_mixer_selem_regopt mixer_options;
 
 /* mixer bar channel : left or right */
 #define MIXER_CHN_LEFT		0
@@ -535,13 +538,13 @@ mixer_convert_volume(snd_mixer_elem_t *elem,
 /* update enum list */
 static void update_enum_list(snd_mixer_elem_t *elem, int chn, int delta)
 {
-  int eidx;
+  unsigned int eidx;
   if (snd_mixer_selem_get_enum_item(elem, chn, &eidx) < 0)
     return;
   if (delta < 0) {
-    eidx--;
-    if (eidx < 0)
+    if (eidx == 0)
       return;
+    eidx--;
   } else {
     int items = snd_mixer_selem_get_enum_items(elem);
     if (items < 0)
@@ -721,14 +724,9 @@ static void draw_blank(int x, int y, int lines)
     mvaddstr (y - i, x, "         ");
 }
 
-/* show the information of the focused item */
-static void display_item_info(int elem_index, snd_mixer_selem_id_t *sid, char *extra_info)
+/* show the current view mode */
+static void display_view_info(void)
 {
-  char string[64], idxstr[10];
-  int idx;
-  int i, xlen = mixer_max_x - 8;
-  if (xlen > sizeof(string) - 1)
-    xlen = sizeof(string) - 1;
   mixer_dc (DC_PROMPT);
   mvaddstr (3, 2, "View:  Playback  Capture  All ");
   mixer_dc (DC_TEXT);
@@ -743,6 +741,16 @@ static void display_item_info(int elem_index, snd_mixer_selem_id_t *sid, char *e
     mvaddstr (3, 27, "[All]");
     break;
   }
+}
+
+/* show the information of the focused item */
+static void display_item_info(int elem_index, snd_mixer_selem_id_t *sid, char *extra_info)
+{
+  char string[64], idxstr[10];
+  int idx;
+  int i, xlen = mixer_max_x - 8;
+  if (xlen > sizeof(string) - 1)
+    xlen = sizeof(string) - 1;
   mixer_dc (DC_PROMPT);
   mvaddstr (4, 2, "Item: ");
   mixer_dc (DC_TEXT);
@@ -788,11 +796,11 @@ static void display_enum_list(snd_mixer_elem_t *elem, int y, int x)
 {
   int cury, ch, err;
 
-  draw_blank(x, y, mixer_cbar_height + 5);
+  draw_blank(x, y, mixer_cbar_height + (mixer_view == VIEW_PLAYBACK ? 5 : 6));
   
   cury = y - 4;
   for (ch = 0; ch < 2; ch++) {
-    int eidx, ofs;
+    unsigned int eidx, ofs;
     char tmp[9];
     err = snd_mixer_selem_get_enum_item(elem, ch, &eidx);
     if (err < 0)
@@ -952,6 +960,7 @@ mixer_update_cbar (int elem_index)
   /* update the focused full bar name
    */
   if (elem_index == mixer_focus_elem) {
+    char tmp[50];
     /* control muted? */
     swl = swr = 1;
     extra_info = "";
@@ -962,25 +971,62 @@ mixer_update_cbar (int elem_index)
 	snd_mixer_selem_get_playback_switch(elem, chn_right, &swr);
       extra_info = !swl && !swr ? " [Off]" : "";
     }
-    if (mixer_type[elem_index] == MIXER_ELEM_ENUM) {
-      while (1) {
-	unsigned int eidx, err, length;
-        char tmp[50];
-	tmp[0]=' ';
-	tmp[1]='[';
-        err = snd_mixer_selem_get_enum_item(elem, 0, &eidx);
-        if (err < 0)
-          break;
-        if (snd_mixer_selem_get_enum_item_name(elem, eidx, sizeof(tmp) - 3, tmp+2) < 0)
-          break;
-        tmp[48] = 0;
+    if (type == MIXER_ELEM_ENUM) {
+      /* FIXME: should show the item names of secondary and later channels... */
+      unsigned int eidx, length;
+      tmp[0]=' ';
+      tmp[1]='[';
+      if (! snd_mixer_selem_get_enum_item(elem, 0, &eidx) &&
+	  ! snd_mixer_selem_get_enum_item_name(elem, eidx, sizeof(tmp) - 3, tmp+2)) {
+	tmp[sizeof(tmp)-2] = 0;
 	length=strlen(tmp);
 	tmp[length]=']';
 	tmp[length+1]=0;
-        extra_info = tmp;
-	break;
+	extra_info = tmp;
       }
     }
+    if (type != MIXER_ELEM_CAPTURE && snd_mixer_selem_has_playback_volume(elem)) {
+	long vdbleft, vdbright;
+	unsigned int length;
+	if (!snd_mixer_selem_get_playback_dB(elem, chn_left, &vdbleft)) {
+		if ((chn_right != SND_MIXER_SCHN_UNKNOWN) &&
+		     (!snd_mixer_selem_get_playback_dB(elem, chn_right, &vdbright))) {
+			float dbvol1, dbvol2;
+			dbvol1=(float)vdbleft/100;
+			dbvol2=(float)vdbright/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f, %3.2f]",dbvol1, dbvol2);
+		} else {
+			float dbvol1;
+			dbvol1=(float)vdbleft/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f]",dbvol1);
+		}
+		tmp[sizeof(tmp)-2] = 0;
+		length=strlen(tmp);
+		tmp[length+1]=0;
+		extra_info = tmp;
+	}
+    }
+    if (type == MIXER_ELEM_CAPTURE && snd_mixer_selem_has_capture_volume(elem)) {
+	long vdbleft, vdbright;
+	unsigned int length;
+	if (!snd_mixer_selem_get_capture_dB(elem, chn_left, &vdbleft)) {
+		if ((chn_right != SND_MIXER_SCHN_UNKNOWN) &&
+		     (!snd_mixer_selem_get_capture_dB(elem, chn_right, &vdbright))) {
+			float dbvol1, dbvol2;
+			dbvol1=(float)vdbleft/100;
+			dbvol2=(float)vdbright/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f, %3.2f]",dbvol1, dbvol2);
+		} else {
+			float dbvol1;
+			dbvol1=(float)vdbleft/100;
+    			snprintf(tmp, 48, " [dB gain=%3.2f]",dbvol1);
+		}
+		tmp[sizeof(tmp)-2] = 0;
+		length=strlen(tmp);
+		tmp[length+1]=0;
+		extra_info = tmp;
+	}
+    }
     display_item_info(elem_index, sid, extra_info);
   }
 
@@ -1078,6 +1124,7 @@ mixer_update_cbars (void)
   static int o_y = 0;
   int i, x, y;
   
+  display_view_info();
   if (!mixer_cbar_get_pos (mixer_focus_elem, &x, &y))
     {
       if (mixer_focus_elem < mixer_first_vis_elem)
@@ -1571,9 +1618,9 @@ mixer_init (void)
    */
   if ((err = snd_mixer_open (&mixer_handle, 0)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_open", err);
-  if ((err = snd_mixer_attach (mixer_handle, card_id)) < 0)
+  if (mixer_level == 0 && (err = snd_mixer_attach (mixer_handle, card_id)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_attach", err);
-  if ((err = snd_mixer_selem_register (mixer_handle, NULL, NULL)) < 0)
+  if ((err = snd_mixer_selem_register (mixer_handle, mixer_level > 0 ? &mixer_options : NULL, NULL)) < 0)
     mixer_abort (ERR_FCN, "snd_mixer_selem_register", err);
   snd_mixer_set_callback (mixer_handle, mixer_event);
   if ((err = snd_mixer_load (mixer_handle)) < 0)
@@ -1932,6 +1979,7 @@ mixer_iteration (void)
       /* ignore */
       break;
     case 27:	/* Escape */
+    case KEY_F (10):
       finished = 1;
       key = 0;
       break;
@@ -1963,11 +2011,6 @@ mixer_iteration (void)
         mixer_view = mixer_view_saved = VIEW_PLAYBACK;
         mixer_changed_state=1;
         mixer_reinit ();
-        if (mixer_n_view_elems == 0) {
-          mixer_view = mixer_view_saved = VIEW_CAPTURE;
-          mixer_changed_state=1;
-          mixer_reinit ();
-        }
       } 
       key = 0;
       break;
@@ -1978,11 +2021,6 @@ mixer_iteration (void)
         mixer_view = mixer_view_saved = VIEW_CAPTURE;
         mixer_changed_state=1;
         mixer_reinit ();
-        if (mixer_n_view_elems == 0) {
-          mixer_view = mixer_view_saved = VIEW_PLAYBACK;
-          mixer_changed_state=1;
-          mixer_reinit ();
-        }
       } 
       key = 0;
       break;
@@ -2190,7 +2228,10 @@ mixer_iteration (void)
   if (old_view != mixer_view)
     mixer_clear (FALSE);
   
-  mixer_focus_elem = CLAMP (mixer_focus_elem, 0, mixer_n_view_elems - 1);
+  if (! mixer_n_view_elems)
+    mixer_focus_elem = 0;
+  else
+    mixer_focus_elem = CLAMP (mixer_focus_elem, 0, mixer_n_view_elems - 1);
   
   return finished;
 }
@@ -2207,7 +2248,7 @@ static void
 mixer_signal_handler (int signal)
 {
   if (signal != SIGSEGV)
-    mixer_abort (ERR_SIGNAL, sys_siglist[signal], 0);
+    mixer_abort (ERR_SIGNAL, strsignal(signal), 0);
   else
     {
       fprintf (stderr, "\nSegmentation fault.\n");
@@ -2225,14 +2266,14 @@ main (int    argc,
    */
   do
     {
-      opt = getopt (argc, argv, "c:D:shgV:");
+      opt = getopt (argc, argv, "c:D:shgV:a:");
       switch (opt)
 	{
 	case '?':
 	case 'h':
-	  fprintf (stderr, "%s v%s\n", PRGNAME_UPPER, VERSION);
-	  fprintf (stderr, "Usage: %s [-h] [-c <card: 0...7>] [-D <mixer device>] [-g] [-s] [-V <view>]\n", PRGNAME);
-	  mixer_abort (ERR_NONE, "", 0);
+	  printf ("%s v%s\n", PRGNAME_UPPER, VERSION);
+	  printf ("Usage: %s [-h] [-c <card: 0...7>] [-D <mixer device>] [-g] [-s] [-V <view>] [-a <abst>]\n", PRGNAME);
+	  return 1;
 	case 'c':
 	  {
 	    int i = snd_card_get_index(optarg);
@@ -2261,16 +2302,31 @@ main (int    argc,
 	  else
 	    mixer_view = VIEW_CHANNELS;
 	  break;
+	case 'a':
+	  mixer_level = 1;
+	  memset(&mixer_options, 0, sizeof(mixer_options));
+	  mixer_options.ver = 1;
+	  if (!strcmp(optarg, "none"))
+	    mixer_options.abstract = SND_MIXER_SABSTRACT_NONE;
+	  else if (!strcmp(optarg, "basic"))
+	    mixer_options.abstract = SND_MIXER_SABSTRACT_BASIC;
+	  else {
+	    fprintf(stderr, "Select correct abstraction level (none or basic)...\n");
+	    mixer_abort (ERR_NONE, "", 0);
+	  }
+	  break;
 	}
     }
   while (opt > 0);
   
+  mixer_options.device = card_id;
+  
   /* initialize mixer
    */
   mixer_init ();
   mixer_reinit ();
   
-  if (mixer_n_view_elems == 0) {
+  if (mixer_n_elems == 0) {
     fprintf(stderr, "No mixer elems found\n");
     mixer_abort (ERR_NONE, "", 0);
   }
@@ -2321,4 +2377,4 @@ main (int    argc,
   while (!mixer_iteration ());
   
   mixer_abort (ERR_NONE, "", 0);
-};
+}
