@@ -86,7 +86,7 @@ static snd_pcm_stream_t stream = SND_PCM_STREAM_PLAYBACK;
 static int mmap_flag = 0;
 static int interleaved = 1;
 static int nonblock = 0;
-static char *audiobuf = NULL;
+static u_char *audiobuf = NULL;
 static snd_pcm_uframes_t chunk_size = 0;
 static unsigned period_time = 0;
 static unsigned buffer_time = 0;
@@ -124,11 +124,12 @@ struct fmt_capture {
 	void (*start) (int fd, size_t count);
 	void (*end) (int fd);
 	char *what;
+	long long max_filesize;
 } fmt_rec_table[] = {
-	{	NULL,		end_raw,	N_("raw data")	},
-	{	begin_voc,	end_voc,	N_("VOC")	},
-	{	begin_wave,	end_wave,	N_("WAVE")	},
-	{	begin_au,	end_au,		N_("Sparc Audio")}
+	{	NULL,		end_raw,	N_("raw data"),		LLONG_MAX },
+	{	begin_voc,	end_voc,	N_("VOC"),		16000000 },
+	{	begin_wave,	end_wave,	N_("WAVE"),		2000000000 },
+	{	begin_au,	end_au,		N_("Sparc Audio"),	LLONG_MAX }
 };
 
 #if __GNUC__ > 2 || (__GNUC__ == 2 && __GNUC_MINOR__ >= 95)
@@ -303,6 +304,8 @@ static void version(void)
 
 static void signal_handler(int sig)
 {
+	if (verbose==2)
+		putchar('\n');
 	if (!quiet_mode)
 		fprintf(stderr, _("Aborted by signal %s...\n"), strsignal(sig));
 	if (stream == SND_PCM_STREAM_CAPTURE) {
@@ -561,7 +564,7 @@ int main(int argc, char *argv[])
 	chunk_size = 1024;
 	hwparams = rhwparams;
 
-	audiobuf = (char *)malloc(1024);
+	audiobuf = (u_char *)malloc(1024);
 	if (audiobuf == NULL) {
 		error(_("not enough memory"));
 		return 1;
@@ -603,6 +606,8 @@ int main(int argc, char *argv[])
 		else
 			capturev(&argv[optind], argc - optind);
 	}
+	if (verbose==2)
+		putchar('\n');
 	snd_pcm_close(handle);
 	free(audiobuf);
 	snd_output_close(log);
@@ -652,7 +657,7 @@ static int test_vocfile(void *buffer)
  * helper for test_wavefile
  */
 
-size_t test_wavefile_read(int fd, char *buffer, size_t *size, size_t reqsize, int line)
+size_t test_wavefile_read(int fd, u_char *buffer, size_t *size, size_t reqsize, int line)
 {
 	if (*size >= reqsize)
 		return *size;
@@ -677,10 +682,10 @@ size_t test_wavefile_read(int fd, char *buffer, size_t *size, size_t reqsize, in
  *                            == 0 if not
  * Value returned is bytes to be discarded.
  */
-static ssize_t test_wavefile(int fd, char *_buffer, size_t size)
+static ssize_t test_wavefile(int fd, u_char *_buffer, size_t size)
 {
 	WaveHeader *h = (WaveHeader *)_buffer;
-	char *buffer = NULL;
+	u_char *buffer = NULL;
 	size_t blimit = 0;
 	WaveFmtBody *f;
 	WaveChunkHeader *c;
@@ -911,8 +916,15 @@ static void set_params(void)
 	assert(err >= 0);
 	if ((float)rate * 1.05 < hwparams.rate || (float)rate * 0.95 > hwparams.rate) {
 		if (!quiet_mode) {
+			char plugex[64];
 			fprintf(stderr, _("Warning: rate is not accurate (requested = %iHz, got = %iHz)\n"), rate, hwparams.rate);
-			fprintf(stderr, _("         please, try the plug plugin (-Dplug:%s)\n"), snd_pcm_name(handle));
+			if (strchr(snd_pcm_name(handle), ':'))
+				*plugex = 0;
+			else
+				snprintf(plugex, sizeof(plugex), "(-Dplug:%s)",
+					 snd_pcm_name(handle));
+			fprintf(stderr, _("         please, try the plug plugin %s\n"),
+				plugex);
 		}
 	}
 	rate = hwparams.rate;
@@ -1183,17 +1195,47 @@ static void compute_max_peak(u_char *data, size_t count)
 	max = 1 << (bits_per_sample-1);
 	if (max <= 0)
 		max = 0x7fffffff;
-	printf(_("Max peak (%li samples): 0x%08x "), (long)ocount, max_peak);
+
 	if (bits_per_sample > 16)
 		perc = max_peak / (max / 100);
 	else
 		perc = max_peak * 100 / max;
-	for (val = 0; val < 20; val++)
-		if (val <= perc / 5)
-			putc('#', stdout);
-		else
-			putc(' ', stdout);
-	printf(" %i%%\n", perc);
+
+	if(verbose<=2) {
+		static int maxperc=0;
+		static time_t t=0;
+		const time_t tt=time(NULL);
+		if(tt>t) {
+			t=tt;
+			maxperc=0;
+		}
+		if(perc>maxperc)
+			maxperc=perc;
+
+		putchar('\r');
+		for (val = 0; val <= perc / 2 && val < 50; val++)
+			putchar('#');
+		for (; val < maxperc / 2 && val < 50; val++)
+			putchar(' ');
+		putchar('+');
+		for (++val; val < 50; val++)
+			putchar(' ');
+
+		printf("| %02i%%", maxperc);
+		if (perc>99)
+			printf(_(" !clip  "));
+
+		fflush(stdout);
+	}
+	else if(verbose==3) {
+		printf(_("Max peak (%li samples): 0x%08x "), (long)ocount, max_peak);
+		for (val = 0; val < 20; val++)
+			if (val <= perc / 5)
+				putchar('#');
+			else
+				putchar(' ');
+		printf(" %i%%\n", perc);
+	}
 }
 
 /*
@@ -1383,9 +1425,9 @@ static ssize_t voc_pcm_write(u_char *data, size_t count)
 static void voc_write_silence(unsigned x)
 {
 	unsigned l;
-	char *buf;
+	u_char *buf;
 
-	buf = (char *) malloc(chunk_bytes);
+	buf = (u_char *) malloc(chunk_bytes);
 	if (buf == NULL) {
 		error(_("can't allocate buffer for silence"));
 		return;		/* not fatal error */
@@ -1951,33 +1993,6 @@ void playback_go(int fd, size_t loaded, off64_t count, int rtype, char *name)
 	snd_pcm_drain(handle);
 }
 
-/* capturing raw data, this proc handels WAVE files and .VOCs (as one block) */
-
-void capture_go(int fd, off64_t count, int rtype, char *name)
-{
-	size_t c;
-	off64_t cur;
-	ssize_t r, err;
-
-	header(rtype, name);
-	set_params();
-
-	do {
-		for (cur = count; cur > 0; cur -= r) {
-			c = (cur <= chunk_bytes) ? cur : chunk_bytes;
-			c = c * 8 / bits_per_frame;
-			if ((size_t)(r = pcm_read(audiobuf, c)) != c)
-				break;
-			r = r * bits_per_frame / 8;
-			if ((err = write(fd, audiobuf, r)) != r) {
-				perror(name);
-				exit(EXIT_FAILURE);
-			}
-			if (err > 0)
-				fdcount += err;
-		}
-	} while (rtype == FORMAT_RAW && !timelimit);
-}
 
 /*
  *  let's play or capture it (capture_type says VOC/WAVE/raw)
@@ -2038,20 +2053,15 @@ static void playback(char *name)
 		close(fd);
 }
 
-static void capture(char *name)
+static void capture(char *name_)
 {
-	pbrec_count = LLONG_MAX;
-	if (!name || !strcmp(name, "-")) {
-		fd = fileno(stdout);
-		name = "stdout";
-	} else {
-		remove(name);
-		if ((fd = open64(name, O_WRONLY | O_CREAT, 0644)) == -1) {
-			perror(name);
-			exit(EXIT_FAILURE);
-		}
-	}
-	fdcount = 0;
+	int tostdout=0;		/* boolean which describes output stream */
+	int filecount=0;	/* number of files written */
+	char *name=name_;	/* current filename */
+	char namebuf[PATH_MAX+1];
+	off64_t cur;		/* number of bytes to capture */
+
+	/* get number of bytes to capture */
 	pbrec_count = calc_count();
 	/* WAVE-file should be even (I'm not sure), but wasting one byte
 	   isn't a problem (this can only be in 8 bit mono) */
@@ -2061,10 +2071,107 @@ static void capture(char *name)
 		pbrec_count -= pbrec_count % 2;
 	if (pbrec_count == 0)
 		pbrec_count -= 2;
-	if (fmt_rec_table[file_type].start)
-		fmt_rec_table[file_type].start(fd, pbrec_count);
-	capture_go(fd, pbrec_count, file_type, name);
-	fmt_rec_table[file_type].end(fd);
+
+	cur = pbrec_count;
+
+	/* display verbose output to console */
+	header(file_type, name);
+
+	/* setup sound hardware */
+	set_params();
+
+	/* write to stdout? */
+	if (!name || !strcmp(name, "-")) {
+		fd = fileno(stdout);
+		name = "stdout";
+		tostdout=1;
+		fdcount = 0;
+	}
+
+	do {
+		/* open a file to write */
+		if(!tostdout) {
+
+			/* upon the second file we start the numbering scheme */
+			if(filecount)
+			{
+				/* get a copy of the original filename */
+				char *s;
+				char buf[PATH_MAX+1];
+				strncpy(buf, name_, sizeof(buf));
+
+				/* separate extension from filename */
+				s=buf+strlen(buf);
+				while(s>buf && *s!='.' && *s!='/')
+					--s;
+				if(*s=='.')
+					*s++=0;
+				else if(*s=='/')
+					s=buf+strlen(buf);
+
+				/* upon first jump to this if block rename the first file */
+				if(filecount==1) {
+					if(*s)
+						snprintf(namebuf, sizeof(namebuf), "%s-01.%s", buf, s);
+					else
+						snprintf(namebuf, sizeof(namebuf), "%s-01", buf);
+					remove(namebuf);
+					rename(name, namebuf);
+					filecount=2;
+				}
+
+				/* name of the current file */
+				if(*s)
+					snprintf(namebuf, sizeof(namebuf), "%s-%02i.%s", buf, filecount, s);
+				else
+					snprintf(namebuf, sizeof(namebuf), "%s-%02i", buf, filecount);
+				name=namebuf;
+
+			}
+
+			/* open a new file */
+			remove(name);
+			if ((fd = open64(name, O_WRONLY | O_CREAT, 0644)) == -1) {
+				perror(name);
+				exit(EXIT_FAILURE);
+			}
+			fdcount = 0;
+			filecount++;
+		}
+
+		/* setup sample header */
+		if (fmt_rec_table[file_type].start)
+			fmt_rec_table[file_type].start(fd, pbrec_count);
+
+		/* capture */
+		do {
+			ssize_t r=0;
+			for (; cur > 0 && fdcount<fmt_rec_table[file_type].max_filesize; cur -= r) {
+				ssize_t err;
+				size_t c = (cur <= chunk_bytes) ? cur : chunk_bytes;
+				c = c * 8 / bits_per_frame;
+				if ((size_t)(r = pcm_read(audiobuf, c)) != c)
+					break;
+				r = r * bits_per_frame / 8;
+				if ((err = write(fd, audiobuf, r)) != r) {
+					perror(name);
+					exit(EXIT_FAILURE);
+				}
+				if (err > 0)
+					fdcount += err;
+			}
+			/* exit conditions:
+			   1) format_raw and a timelimit
+			   2) all requested samples/bytes have been captured (cur>0)
+			   3) filesize threshold was reached (fdcount<wrthreshold)
+			*/
+		} while (file_type == FORMAT_RAW && !timelimit && cur>0 && fdcount<fmt_rec_table[file_type].max_filesize);
+
+		/* finish sample container */
+		fmt_rec_table[file_type].end(fd);
+
+		/* repeat the loop when format is raw without timelimit or filesize threshold was reached */
+	} while((file_type == FORMAT_RAW && !timelimit) || fdcount>=fmt_rec_table[file_type].max_filesize);
 }
 
 void playbackv_go(int* fds, unsigned int channels, size_t loaded, off64_t count, int rtype, char **names)
